{% extends "admin/change_form.html" %}
{% load i18n admin_urls %}

{% block object-tools-items %}
    <li>
        <a href="{% url opts|admin_urlname:'calculate-values' original.pk|admin_urlquote %}">{% trans "Calculate Values" %}</a>
    </li>
    <li>
        <a href="{% url opts|admin_urlname:'process-gpx-data' original.pk|admin_urlquote %}">{% trans "Process GPX data" %}</a>
    </li>
    {{ block.super }}
{% endblock %}
