{% extends "admin/change_list.html" %}
{% load i18n admin_urls %}

{% block object-tools-items %}
    <li>
        <a href="upload/">Upload GPX file</a>
    </li>
    {{ block.super }}
{% endblock %}
